class SPK_Camera extends CameraActor DependsOn( SPK_Path );

Enum SPK_CameraMode
{

	SPK_CAMERAMODE_ABOVE,
	SPK_CAMERAMODE_PLAY,
	SPK_CAMERAMODE_ORBIT

};

var Actor Track;
Var SPK_CameraMode CameraMode;
Var Float Strafe;

Var Vector PreviousTarget;
Var Float SinceSpawn;

Var Array<Class<SPK_EnvironmentActor> > EnvironmentActorClasses;

/* Don't update the camera for the next frame; used to work around a Tick() execution order race condition. */
Var Bool SkipFrame;

Event Tick( Float FrameDelta )
{

	Local Vector Yaw, Position, ShipX, ShipY, ShipZ;
	Local Rotator Turn;
	Local SPK_Ship Ship;
	Local Float CameraWidth, CameraDistance;
    Local SPK_EnvironmentActor Spawned;

	switch( CameraMode )
	{

		case SPK_CAMERAMODE_ABOVE:

			if( Track != None )
			{

                If( SkipFrame )
                {

                    SkipFrame = False;

                    Super.Tick( FrameDelta );

                    Return;

                }

				SetLocation( Track.Location + Vect( 0.0f, 0.0f, 640.0f ) );

				Yaw = Vector( Track.Rotation );

				Yaw.Z = 0.0f;

				Turn.Pitch = 0;
				Turn.Roll = 0;
				Turn.Yaw = 16384;

				SetRotation( Rotator( Vect( 0.0f, 0.0f, -640.0f ) ) + Rotator( Yaw ) + Turn );

			}

		break;

		case SPK_CAMERAMODE_PLAY:

			if( Track != None )
			{

				Ship = SPK_Ship( Track );

				if( Ship != None )
				{

                    If( SkipFrame )
                    {

                        SkipFrame = False;

                        Super.Tick( FrameDelta );

                        Return;

                    }

					CameraWidth = sin( FOVAngle * 0.01745329252 ) * Lerp( 480.0f, 320.0f, ( Ship.Speed - Ship.SpeedMin ) / ( Ship.SpeedMax - Ship.SpeedMin ) );

					GetAxes( Ship.Rotation, ShipX, ShipY, ShipZ );

					Strafe = Lerp( Clamp( ( ShipY dot ( Ship.AttachedTo.GetPosition( Ship.AttachedTo.Progress( Ship.Progress, 768.0f ) ) - Ship.Location ) ), -256.0f, 256.0f ), Strafe, 0.1f ** FrameDelta );

					Position = Ship.Location + ( Strafe * ShipY ) + ( CameraWidth * ShipX );

					Yaw = Vector( Track.Rotation );
					Yaw.Z = 0.0f;

					Turn.Pitch = 0;
					Turn.Roll = 0;
					Turn.Yaw = 16384;

					SetLocation( Position + Vect( 0.0f, 0.0f, 640.0f ) );
					SetRotation( Rotator( Vect( 0.0f, 0.0f, -640.0f ) ) + Rotator( Ship.AttachedTo.GetDirectionCustomSeek( Ship.Progress, 512.0f ) * Vect( 1.0f, 1.0f, 0.0f ) ) + Turn );

				}

			}

		break;

	}

    GetAxes( Rotation, ShipX, ShipY, ShipZ );

    Position = Location + ( ShipX * 640.0f );

    SinceSpawn += VSize( Position - PreviousTarget );

    CameraWidth = Sin( FOVAngle * 0.0174532925 ) * 640.0f;

    While( SinceSpawn > 0.0f )
    {

        Spawned = Spawn( EnvironmentActorClasses[ Rand( EnvironmentActorClasses.Length ) ], Self );

        If( Spawned != None )
        {

            CameraDistance = Sin( FOVAngle * 0.0174532925 ) * ( 640.0f - Spawned.ZOffset );

            If( Spawned.bSpawnOnTrack ) Spawned.SetLocation( ( ShipX * -Spawned.ZOffset ) + Position + ( ShipZ * 64.0f * ( ( FRand() - 0.5f ) * 2.0f ) ) - ( ( CameraDistance + Spawned.Radius ) * ShipY ) );
            Else Spawned.SetLocation( ( ShipX * -Spawned.ZOffset ) + Position + ( ( CameraDistance * ShipZ * AspectRatio ) * ( ( FRand() - 0.5f ) * 2.0f ) ) - ( ( CameraDistance + Spawned.Radius ) * ShipY ) );

        }

        SinceSpawn -= 256.0f;

    }

    PreviousTarget = Position;

}

Function Bool IsInView( Vector Position, Float Radius )
{

    Local Vector XAxis, YAxis, ZAxis, Difference;
    Local Float ZDistance;

    GetAxes( Rotation, XAxis, YAxis, ZAxis );

    Difference = Position - Location;

    ZDistance = XAxis Dot Difference;

    If( ZDistance < -Radius ) Return False;

    ZDistance *= Sin( FOVAngle * 0.0174532925 );

    If( ( Abs( ZAxis Dot Difference ) - Radius ) > ( ZDistance * AspectRatio ) ) Return False;
    If( ( Abs( YAxis Dot Difference ) - Radius ) > ZDistance ) Return False;

    Return True;

}

defaultproperties
{

	Strafe = 0.0f;
	Track = None;
	CameraMode = SPK_CAMERAMODE_PLAY;

    SinceSpawn = 0.0f;
    PreviousTarget = ( X = 0.0f, Y = 0.0f, Z = 0.0f );

    EnvironmentActorClasses = ( Class'SPK_EnvironmentNebula', Class'SPK_EnvironmentAsteroid' );

    SkipFrame = False;

	Begin Object Class=ParticleSystemComponent Name=ParticleSystemComponent0

		Template = ParticleSystem'SPK_Environment.Dust';

	End Object

	Components.Add( ParticleSystemComponent0 );

	bConstrainAspectRatio = False;
	bNoDelete = False;

}
