Class SPK_EnvironmentActor Extends Actor;

/* Used for crude frustum culling. */
Var Float Radius;

/* Random mesh selection variables. */
Var Bool bUsesMeshPrefix;
Var String MeshPrefix;
Var Int SuffixMin, SuffixMax;
Var StaticMesh MeshNoPrefix;

/* Wandering for asteroids/space junk/etc. */
Var Bool bWander, bWanderZ;
Var Vector WanderSpeed;
Var Float WanderDamping, WanderLimit;
Var Int WanderForce;

/* Spin. */
Var Bool bSpin;
Var Rotator SpinSpeed;
Var Float SpinDamping, SpinLimit;
Var Int SpinForce;

Var StaticMeshComponent Mesh;

/* Does this environment actor need to appear on the track? */
Var Bool bSpawnOnTrack;

/* Offset relative to the track on the Z axis to place; useful for parallax. */
Var Float ZOffset;

Event PostBeginPlay()
{

	Local StaticMesh ToUse;

	ToUse = GetMesh();

	If( ToUse == None ) `Warn( Self$" failed to choose a static mesh." ); Else
	{

		Mesh.SetStaticMesh( ToUse );

	}

    If( bWander ) WanderSpeed = ( ( VRand() * 2.0f ) - Vect( 1.0f, 1.0f, 1.0f ) ) * WanderLimit;

    If( bSpin )
    {

        SpinSpeed.Pitch = Lerp( -SpinLimit, SpinLimit, FRand() );
        SpinSpeed.Yaw = Lerp( -SpinLimit, SpinLimit, FRand() );
        SpinSpeed.Roll = Lerp( -SpinLimit, SpinLimit, FRand() );

    }

	Super.PostBeginPlay();

}

Function Bool EvaluateNoise( Float Current, Float Limit )
{

	If( Rand( 512 ) < ( ( ( Current + Limit ) / ( Limit + Limit ) ) * 512.0f ) ) Return True;

	Return False;

}

Event Tick( Float FrameDelta )
{

	Local Float Strength;
    Local SPK_Camera OwnerCast;
    Local SPK_Game Game;

    OwnerCast = SPK_Camera( Owner );

    If( OwnerCast == None )
    {

        `Warn( Self$" was not spawned by a SPK_Camera. ("$Owner$")." );

        Destroy();

        Return;

    }

    If( !OwnerCast.IsInView( Location, Radius * 2.0f ) )
    {

        Destroy();

        Return;

    }

	If( bWander )
	{

		Strength = FrameDelta * WanderForce;

		If( EvaluateNoise( WanderSpeed.X, WanderLimit ) ) WanderSpeed.X -= Strength; Else WanderSpeed.X += Strength;
		If( EvaluateNoise( WanderSpeed.Y, WanderLimit ) ) WanderSpeed.Y -= Strength; Else WanderSpeed.Y += Strength;

        If( bSpawnOnTrack )
        {

            Game = SPK_Game( WorldInfo.Game );

            If( Game != None ){  If( Game.PlayerShip != None ) {If( Game.PlayerShip.Location.Z < Location.Z ) WanderSpeed.Z -= Strength; Else WanderSpeed.Z += Strength;} Else {
                                    If( EvaluateNoise( WanderSpeed.Z, WanderLimit ) ) WanderSpeed.Z -= Strength; Else WanderSpeed.Z += Strength; }
                }Else {If( EvaluateNoise( WanderSpeed.Z, WanderLimit ) ) WanderSpeed.Z -= Strength; Else WanderSpeed.Z += Strength;}

        } Else If( EvaluateNoise( WanderSpeed.Z, WanderLimit ) ) WanderSpeed.Z -= Strength; Else WanderSpeed.Z += Strength;

		WanderSpeed *= WanderDamping ** FrameDelta;

		WanderSpeed.X = FClamp( WanderSpeed.X, -WanderLimit, Wanderlimit );
		WanderSpeed.Y = FClamp( WanderSpeed.Y, -WanderLimit, Wanderlimit );
		WanderSpeed.Z = FClamp( WanderSpeed.Z, -WanderLimit, Wanderlimit );

		If( bWanderZ ) SetLocation( Location + ( WanderSpeed * FrameDelta ) ); Else SetLocation( Location + ( WanderSpeed * FrameDelta * Vect( 1.0f, 1.0f, 0.0f ) ) );

	}

	If( bSpin )
	{

		Strength = FrameDelta * SpinForce;

		If( EvaluateNoise( SpinSpeed.Pitch, SpinLimit ) ) SpinSpeed.Pitch -= Strength; Else SpinSpeed.Pitch += Strength;
		If( EvaluateNoise( SpinSpeed.Yaw, SpinLimit ) ) SpinSpeed.Yaw -= Strength; Else SpinSpeed.Yaw += Strength;
		If( EvaluateNoise( SpinSpeed.Roll, SpinLimit ) ) SpinSpeed.Roll -= Strength; Else SpinSpeed.Roll += Strength;

		SpinSpeed *= SpinDamping ** FrameDelta;

		SpinSpeed.Pitch = FClamp( SpinSpeed.Pitch, -SpinLimit, Spinlimit );
		SpinSpeed.Yaw = FClamp( SpinSpeed.Yaw, -SpinLimit, Spinlimit );
		SpinSpeed.Roll = FClamp( SpinSpeed.Roll, -SpinLimit, Spinlimit );

		SetRotation( Rotation + ( SpinSpeed * FrameDelta ) );

	}

	Super.Tick( FrameDelta );

}

Function StaticMesh GetMesh()
{

	Local String Reference;

	If( bUsesMeshPrefix )
	{

		Reference = "StaticMesh'"$MeshPrefix$"_"$( SuffixMin + Rand( SuffixMax - SuffixMin ) )$"'";

		Return StaticMesh( DynamicLoadObject( Reference, Class'StaticMesh', False ) );

	}

	Return MeshNoPrefix;

}

DefaultProperties
{

	bUsesMeshPrefix = False;
	MeshPrefix = "_";
	MeshNoPrefix = None;
	SuffixMin = 0;
	SuffixMax = 0;

    bWander = False;
    bWanderZ = False;
    WanderSpeed = ( X = 0.0f, Y = 0.0f, Z = 0.0f );
    WanderDamping = 0.0f;
    WanderLimit = 0.0f;
    WanderForce = 0;

    bSpin = False;
    SpinSpeed = ( Pitch = 0, Yaw = 0, Roll = 0 );
    SpinDamping = 0.0f;
    SpinLimit = 0.0f;
    SpinForce = 0;

	Radius = 0.0f;

    bSpawnOnTrack = False;
    ZOffset = 0.0f;

	Begin Object Class=StaticMeshComponent Name=DisplayMesh

	End Object

	Components.Add( DisplayMesh );

	Mesh = DisplayMesh;

	bStatic = False;

}
