class SPK_Game extends GameInfo;

var Class<SPK_Ship> PlayerShipType;

Struct Wave
{

    Var String State;
    Var Float Difficulty;

};

Var Array<Wave> Waves;

Var SPK_Ship PlayerShip;

/* What is the total damage that should be inflicted on a ship to ship collision? */
/* Note this will be divided up according to mass. */
Var Float CollisionDamage;

Var Int Score, Lives;

/* Pick a random state from the array and go to it. */
/* Should be called only at the end of a wave. */
Function ChooseWave()
{

    Local Float ChosenDifficulty, Nearest;
    Local Wave Closest;
    Local Int Iterate;

    If( PlayerShip == None ) GotoState( 'DelayedWave' ); Else
    {

        ChosenDifficulty = Float( PlayerShip.AttachedTo.Vertices.Find( PlayerShip.Progress.Leaving ) ) / Float( PlayerShip.AttachedTo.Vertices.Length ) + Lerp( -0.1f, 0.1f, FRand() );

        Closest = Waves[ 0 ];
        Nearest = Abs( Waves[ 0 ].Difficulty - ChosenDifficulty );

        For( Iterate = 1; Iterate < Waves.Length; Iterate++ )
        {

            If( Abs( Waves[ Iterate ].Difficulty - ChosenDifficulty ) < Nearest )
            {

                Nearest = Abs( Waves[ Iterate ].Difficulty - ChosenDifficulty );
                Closest = Waves[ Iterate ];

            }

        }

        `Log( "State decision: chose random value "$ChosenDifficulty$", closest to the difficulty of state "$Closest.State$", which has a difficulty rating of "$Closest.Difficulty$"." );

        GotoState( Name( Closest.State ) );

    }

}

Auto State NoWave
{

    Begin:

    ChooseWave();

}

State DelayedWave
{

    Begin:

    Sleep( 0.5f );

    ChooseWave();

}

State Finishing
{

    Begin:

        `Log( "This is the endgame." );

        Sleep( 2.0f );

        WorldInfo.ServerTravel( "SPK_Menu" );

}

State WaveDualHelix
{

    Begin:

        `Log( "Now running Dual Helix." );

        WaveSpawn( Class'SPK_SwooperShip', False, 80.0f );

        Sleep( 1.5f );

        WaveSpawn( Class'SPK_SwooperShip', False, -80.0f );

        Sleep( 1.5f );

        WaveSpawn( Class'SPK_SwooperShip', False, 80.0f );

        Sleep( 1.5f );

        WaveSpawn( Class'SPK_SwooperShip', False, -80.0f );

        Sleep( 6.0f );

        ChooseWave();

}

State WaveFlankedBattleship
{

    Begin:

        `Log( "Now running Flanked Battleship." );

        WaveSpawn( Class'SPK_BattleShip', False, 0.0f );

        Sleep( 0.5f );

        WaveSpawn( Class'SPK_QuickrunnerShip', False, 80.0f );

        Sleep( 0.5f );

        WaveSpawn( Class'SPK_QuickrunnerShip', False, -80.0f );

        Sleep( 5.0f );

        ChooseWave();

}

State WaveDualAmbush
{

    Begin:

        `Log( "Now running Dual Ambush." );

        WaveSpawn( Class'SPK_QuickrunnerShip', True, 80.0f );

        Sleep( 0.5f );

        WaveSpawn( Class'SPK_QuickrunnerShip', True, -80.0f );

        Sleep( 5.0f );

        ChooseWave();

}

State WaveScout
{

    Begin:

        `Log( "Now running Scout." );

        WaveSpawn( Class'SPK_QuickrunnerShip', False, 0.0f );

        Sleep( 5.0f );

        ChooseWave();

}

State WaveBattleships
{

    Begin:

        `Log( "Now running Battleships." );

        WaveSpawn( Class'SPK_BattleShip', False, 70.0f );

        WaveSpawn( Class'SPK_BattleShip', False, -70.0f );

        Sleep( 8.0f );

        ChooseWave();

}

State WaveSwarm
{

    Begin:

        `Log( "Now running Swarm." );

        WaveSpawn( Class'SPK_QuickrunnerShip', False, 0.0f );

        Sleep( 0.5f );

        WaveSpawn( Class'SPK_QuickrunnerShip', False, 40.0f );

        Sleep( 0.5f );

        WaveSpawn( Class'SPK_QuickrunnerShip', False, -40.0f );

        Sleep( 0.5f );

        WaveSpawn( Class'SPK_QuickrunnerShip', False, 70.0f );

        Sleep( 0.5f );

        WaveSpawn( Class'SPK_QuickrunnerShip', False, 0.0f );

        Sleep( 0.5f );

        WaveSpawn( Class'SPK_QuickrunnerShip', False, -70.0f );

        Sleep( 5.0f );

        ChooseWave();

}

State WaveBarricade
{

    Begin:

        `Log( "Now running Barricade." );

        WaveSpawn( Class'SPK_SwooperShip', False, 80.0f );

        Sleep( 1.5f );

        WaveSpawn( Class'SPK_SwooperShip', False, -80.0f );

        Sleep( 1.5f );

        WaveSpawn( Class'SPK_QuickrunnerShip', False, 70.0f );

        Sleep( 0.5f );

        WaveSpawn( Class'SPK_QuickrunnerShip', False, 0.0f );

        Sleep( 0.5f );

        WaveSpawn( Class'SPK_QuickrunnerShip', False, -70.0f );

        Sleep( 5.0f );

        ChooseWave();

}

/* Spawn a specified ship either in front or behind at a specified strafe offset.  Used by the wave system. */
Function WaveSpawn( Class<SPK_Ship> ShipType, Bool Ambush, Float Strafe )
{

    Local SPK_Ship Spawned;

    If( PlayerShip == None ) `Warn( "Tried to spawn a ship from a wave when the player was not present." ); Else
    {

        Spawned = Spawn( ShipType );

        If( Spawned == None ) `Warn( "Failed to spawn a ship from a wave." ); Else
        {

            Spawned.AttachedTo = PlayerShip.AttachedTo;
            If( Ambush ) Spawned.Progress = PlayerShip.AttachedTo.Progress( PlayerShip.Progress, -512.0f ); Else Spawned.Progress = PlayerShip.AttachedTo.Progress( PlayerShip.Progress, 1536.0f );
            Spawned.Strafe = Strafe;
            Spawned.Ambush = Ambush;

        }

    }

}

defaultproperties
{

    CollisionDamage = 25;

    PlayerShipType = class'SPK_CruiserShip';

    /* List any states to be used as randomly selected waves here. */
    Waves = ( ( State = "WaveDualHelix", Difficulty = 0.6f ), ( State = "WaveFlankedBattleship", Difficulty = 1.0f ), ( State = "WaveDualAmbush", Difficulty = 0.5f ), ( State = "WaveScout", Difficulty = 0.0f ), ( State = "WaveBattleships", Difficulty = 0.9f ), ( State = "WaveSwarm", Difficulty = 0.65f ), ( State = "WaveBarricade", Difficulty = 0.4f ) );

	PlayerControllerClass = class'SPK_PlayerController';
	HUDType = Class'SPK_GameHUD';

    Lives = 3;
    Score = 0;

}
