Class SPK_GameHUD Extends SPK_HUD;

Var GFxMoviePlayer Health, Score, Paused, Message;
Var GFxObject ScoreText, HealthText, HealthBar, ProgressBar, ChargeBar, LivesCounter, ResumeKey, QuitKey;
Var Int PausedSelected;

Simulated Event PostBeginPlay()
{

	Health = AddClip( SwfMovie'SPK_UI.Health', Align_BottomLeft, False );

	If( Health != None )
	{

		HealthText = Health.GetVariableObject( "_root.Value" );
		HealthBar = Health.GetVariableObject( "_root.Bar_A" );
		ChargeBar = Health.GetVariableObject( "_root.Bar_B" );
		ProgressBar = Health.GetVariableObject( "_root.Bar_C" );

	}

	Score = AddClip( SwfMovie'SPK_UI.Score', Align_TopRight, False );

    If( Score != None )
    {

		ScoreText = Score.GetVariableObject( "_root.Value" );
		LivesCounter = Score.GetVariableObject( "_root.Lives" );

    }

	Super.PostBeginPlay();

}

Singular Event Destroyed()
{

	If( Health != None ) Health.Close();
	If( Score != None ) Score.Close();
    If( Paused != None ) Paused.Close();
    If( Message != None ) Message.Close();

	Super.Destroyed();

}

Event PostRender()
{

	Local Int UIScale;
    Local SPK_Game Game;

    If( PressedCancel )
    {

        If( Paused == None )
        {

            Paused = AddClip( SwfMovie'SPK_UI.Pause', Align_BottomLeft, True );

            If( Paused != None )
            {

                ResumeKey = Paused.GetVariableObject( "_root.ResumeKey" );
                QuitKey = Paused.GetVariableObject( "_root.QuitKey" );

                PausedSelected = 0;

                QuitKey.SetVisible( False );

            }

        }
        Else
        {

            ResumeKey = None;
            QuitKey = None;

            Paused.Close();
            Paused = None;

        }

    }

	UIScale = Canvas.SizeY / 8;

    If( Paused != None )
    {

        Paused.SetViewport( ( Canvas.SizeX / 2 ) - ( UIScale * 1.5 ), UIScale * 3, UIScale * 3, UIScale * 2 );

        If( PressedUp || PressedDown ) PausedSelected = 1 - PausedSelected;

        If( PausedSelected == 0 )
        {

            If( PressedSelect )
            {

                ResumeKey = None;
                QuitKey = None;

                Paused.Close();
                Paused = None;

            }

            QuitKey.SetVisible( False );
            ResumeKey.SetVisible( True );

        }
        Else
        {

            If( PressedSelect ) WorldInfo.ServerTravel( "SPK_Menu" );

            QuitKey.SetVisible( True );
            ResumeKey.SetVisible( False );

        }

    }

    If( Message != None ) Message.SetViewport( Canvas.SizeX - ( UIScale * 2 ), UIScale * 3.5f, UIScale * 4, UIScale );

    Game = SPK_Game( WorldInfo.Game );

    If( Game != None )
    {

        If( Game.PlayerShip != None )
        {

            If( Health != None )
            {

                Health.SetViewport( 0, 0, UIScale, UIScale );
                HealthText.SetString( "text", String( Game.PlayerShip.Health ) );
                HealthBar.GotoAndStopi( 100 - Game.PlayerShip.Health );
                ProgressBar.GotoAndStopi( FClamp( ( Game.PlayerShip.Speed - Game.PlayerShip.SpeedMin ) / ( Game.PlayerShip.SpeedMax - Game.PlayerShip.SpeedMin ) * 100.0f, 0.0f, 100.0f ) );
                ChargeBar.GotoAndStopi( FClamp( ( ( Game.PlayerShip.StrafeAcceleration / Game.PlayerShip.StrafeSpeed ) * 50.0f ) + 50.0f, 0.0f, 100.0f ) );

            }

            If( Score != None )
            {

                Score.SetViewport( Canvas.SizeX - UIScale, Canvas.SizeY - UIScale, UIScale, UIScale );
                ScoreText.SetString( "text", String( Game.Score ) );
                LivesCounter.GotoAndStopi( 4 - Clamp( Game.Lives, 0, 3 ) );

            }

        }

    }

    Super.PostRender();

}

DefaultProperties
{

    PausedSelected = 0;

}
