class SPK_Path extends Actor Placeable;

var array<SPK_PathVertex> Vertices;
var Array<SPK_Edging> Edging;

Struct SPK_PathProgress
{

	Var Float Progress;
	Var SPK_PathVertex Leaving;

};

function SPK_PathProgress Progress( SPK_PathProgress From, float Distance )
{

	local float Rate;
	Local SPK_PathProgress To;

	To = From;

	if( From.Leaving == None ) return To;

	if( Distance > 0 )
	{

		while( Distance > 0.0f )
		{

			if( Vertices.Find( To.Leaving ) >= ( Vertices.Length - 1 ) ) Return To;
			Rate = GetRate( To );

			To.Progress += Distance / Rate;

			if( To.Progress >= 1.0f )
			{

				Distance = ( To.Progress - 1.0f ) * Rate;

				To.Progress = 0.0f;

				To.Leaving = Vertices[ Vertices.Find( To.Leaving ) + 1 ];

			} else Distance = 0.0f;

		}

	}
	else
	{

		while( Distance < 0.0f )
		{

			if( Vertices.Find( To.Leaving ) < 1 ) return To;
			Rate = GetRate( To );

			To.Progress += Distance / Rate;

			if( To.Progress <= 0.0f )
			{

				Distance = ( To.Progress + 1.0f ) * Rate;

				To.Progress = 1.0f;

				To.Leaving = Vertices[ Vertices.Find( To.Leaving ) - 1 ];

			} else Distance = 0.0f;


		}

	}

	Return To;

}

event PostBeginPlay()
{

	local int Generate;
	local Vector Position, LastPosition;
	local Rotator Angle;

	Position = Vect( 0.0f, 0.0f, 1024.0f );
	LastPosition = Position;

	AddVertex( Location + LastPosition );

	for( Generate = 0; Generate < 16; Generate++ )
	{

		Angle.Roll = 0;
		Angle.Pitch = Lerp( -1000, 1000, FRand() );
		Angle.Yaw = Lerp( -6144, 6144, FRand() );

		Angle = Normalize( Rotator( Position - LastPosition ) + Angle );

		Position += Vector( Angle ) * 1024.0f;

		LastPosition = Position;

		AddVertex( Location + Position );

	}

	GenerateInterpolations();
	GenerateEdges();

}

function SPK_PathVertex AddVertex( Vector Position )
{

	local SPK_PathVertex Vertex;

	Vertex = Spawn( class'SPK_PathVertex', , , Position );

	if( Vertex != none ) Vertices[ Vertices.Length ] = Vertex;

	return Vertex;

}

function Vector GetPosition( SPK_PathProgress Progress )
{

	local int ArrayIndex;
	local SPK_PathVertex Entering;
	local Vector Midpoint;

	if( Progress.Leaving == None ) if( Vertices.Length == 0 ) return Location; else return Vertices[ 0 ].Location;

	ArrayIndex = Vertices.Find( Progress.Leaving );

	if( ArrayIndex >= ( Vertices.Length - 1 ) )
	{

		return Progress.Leaving.Location;

	}

	Entering = Vertices[ ArrayIndex + 1 ];

	Midpoint = VLerp( Progress.Leaving.Leave, Entering.Enter, Progress.Progress );

	return VLerp( VLerp( VLerp( Progress.Leaving.Location, Progress.Leaving.Leave, Progress.Progress ), Midpoint, Progress.Progress ), VLerp( Midpoint, VLerp( Entering.Enter, Entering.Location, Progress.Progress ), Progress.Progress ), Progress.Progress );

}

function Float GetRate( SPK_PathProgress Progress )
{

	local int ArrayIndex;
	local SPK_PathVertex Entering;
	local Float ProgressAhead;
	local Vector Midpoint, MidpointAhead;

	if( Progress.Leaving == None ) return 0.0001f;

	ArrayIndex = Vertices.Find( Progress.Leaving );

	if( ArrayIndex >= ( Vertices.Length - 1 ) )
	{

		return 0.0001f;

	}

	Entering = Vertices[ ArrayIndex + 1 ];

	ProgressAhead = Progress.Progress + 0.01f;

	Midpoint = VLerp( Progress.Leaving.Leave, Entering.Enter, Progress.Progress );
	MidpointAhead = VLerp( Progress.Leaving.Leave, Entering.Enter, ProgressAhead );

	return VSize(
		VLerp( VLerp( VLerp( Progress.Leaving.Location, Progress.Leaving.Leave, Progress.Progress ), Midpoint, Progress.Progress ), VLerp( Midpoint, VLerp( Entering.Enter, Entering.Location, Progress.Progress ), Progress.Progress ), Progress.Progress ) -
		VLerp( VLerp( VLerp( Progress.Leaving.Location, Progress.Leaving.Leave, ProgressAhead ), MidpointAhead, ProgressAhead ), VLerp( MidpointAhead, VLerp( Entering.Enter, Entering.Location, ProgressAhead ), ProgressAhead ), ProgressAhead )
																												) * 100.0f;

}

function Vector GetDirectionCustomSeek( SPK_PathProgress Progressor, Float Seek )
{

	return Normal( GetPosition( Progress( Progressor, Seek ) ) - GetPosition( Progressor ) );

}

function Vector GetDirection( SPK_PathProgress Progressor )
{

	return GetDirectionCustomSeek( Progressor, 128.0f );

}

function GenerateInterpolations()
{

	local SPK_PathVertex Vertex, PreviousVertex, EnterVertex;
	local Vector EnterDifference, LeaveDifference, LeaveNormal;
	local Float EnterDistance, LeaveDistance;

	PreviousVertex = none;
	EnterVertex = none;

	foreach Vertices( Vertex )
	{


		if( PreviousVertex != none )
		{

			if( EnterVertex == none )
			{

				EnterDifference.X = 0.0f;
				EnterDifference.Y = 0.0f;
				EnterDifference.Z = 0.0f;

			} else EnterDifference = EnterVertex.Location - PreviousVertex.Location;

			LeaveDifference = Vertex.Location - PreviousVertex.Location;

			LeaveNormal = LeaveDifference - EnterDifference;

			if( VSizeSq( LeaveNormal ) < 0.0001f )
			{

				PreviousVertex.Enter = PreviousVertex.Location + EnterDifference;
				PreviousVertex.Leave = PreviousVertex.Location + LeaveDifference;

			}
			else
			{

				LeaveNormal = Normal( LeaveNormal );

				EnterDistance = EnterDifference dot -LeaveNormal;
				LeaveDistance = LeaveDifference dot LeaveNormal;

				EnterDistance *= Lerp( 1.0f, 0.25f, FClamp( ( ( Normal( EnterDifference ) dot -LeaveNormal ) * 2.0f ) - 1.0f, 0.0f, 1.0f ) );
				LeaveDistance *= Lerp( 1.0f, 0.25f, FClamp( ( ( Normal( LeaveDifference ) dot LeaveNormal ) * 2.0f ) - 1.0f, 0.0f, 1.0f ) );

				LeaveDistance = Min( LeaveDistance, EnterDistance );

				PreviousVertex.Enter = PreviousVertex.Location + ( LeaveDistance * -LeaveNormal );
				PreviousVertex.Leave = PreviousVertex.Location + ( LeaveDistance * LeaveNormal );

			}

			PreviousVertex.Updatesprites();

		}

		EnterVertex = PreviousVertex;
		PreviousVertex = Vertex;

	}

	/* Handle short paths as gracefully as possible. */
	if( PreviousVertex != none )
	{

		if( EnterVertex == none )
		{

			PreviousVertex.Enter.X = 0.0f;
			PreviousVertex.Enter.Y = 0.0f;
			PreviousVertex.Enter.Z = 0.0f;

			PreviousVertex.Leave = PreviousVertex.Enter;

		}
		else
		{

			PreviousVertex.Enter = VLerp( PreviousVertex.Location, EnterVertex.Location, 0.5f );

			PreviousVertex.Leave.X = 0.0f;
			PreviousVertex.Leave.Y = 0.0f;
			PreviousVertex.Leave.Z = 0.0f;

		}

	}

}

Function GenerateEdges()
{

	Local SPK_PathProgress Progression;
	Local SPK_Edging Edge;
	Local Vector Position, OldPosition, Perpendicular, Left, Right, OldLeft, OldRight;
	Local Int Ready;

	if( Vertices.Length > 0 )
	{

		Ready = 0;

		Progression.Progress = 0.0f;
		Progression.Leaving = Vertices[ 0 ];

		While( Vertices.Find( Progression.Leaving ) < ( Vertices.Length - 1 ) )
		{

			Position = GetPosition( Progression );

			Progression = Progress( Progression, 128.0f );

			Perpendicular = Normal( Vect( 0.0f, 0.0f, 1.0f ) Cross ( Position - OldPosition ) ) * 128.0f;

			Left = Position + Perpendicular;
			Right = Position - Perpendicular;

			if( Ready == 2 )
			{

				Edge = Spawn( Class'SPK_Edging' );

				Edge.SetLocation( OldLeft );

				Edge.SetRotation( Rotator( Left - OldLeft ) );

				OldPosition.X = VSize( Left - OldLeft ) * 0.5f;
				OldPosition.Y = 32.0f;
				OldPosition.Z = 32.0f;

				Edge.Mesh.SetScale3D( OldPosition );

				Edge = Spawn( Class'SPK_Edging' );

				Edge.SetLocation( OldRight );

				Edge.SetRotation( Rotator( Right - OldRight ) );

				OldPosition.X = VSize( Right - OldRight ) * 0.5f;
				OldPosition.Y = 32.0f;
				OldPosition.Z = 32.0f;

				Edge.Mesh.SetScale3D( OldPosition );

			} Else Ready++;

			OldLeft = Left;
			OldRight = Right;
			OldPosition = Position;

		}

	}

}

defaultproperties
{


	Begin Object Class=SpriteComponent Name=SpriteComponent

		Sprite=Texture2D'SPK_Sprites.PathDebug'
		HiddenGame=false
		HiddenEditor=false
		AlwaysLoadOnClient=False
		AlwaysLoadOnServer=False

	End Object

	Components.Add( SpriteComponent )

}
