class SPK_PathProjectile extends SPK_PathObject;

var Vector ProjectileVelocity;
var Float Thrust, Damping, InitialThrust;

Var Float FizzleTime;
Var Bool Detonated;

Var SoundCue FireSound, HitSound, FizzleSound;
Var ParticleSystemComponent HitEffect, FizzleEffect;

Var Int Yield;

Var Bool IgnoresInvincibility;

event Tick( float FrameDelta )
{

	ProjectileVelocity *= Damping ** FrameDelta;
	ProjectileVelocity += Normal( ProjectileVelocity ) * Thrust * FrameDelta;

	Strafe += FrameDelta * ProjectileVelocity.Y;

	Progress = AttachedTo.Progress( Progress, FrameDelta * ProjectileVelocity.X );

    If( Detonated )
    {

        If( ( HitEffect.bWasCompleted ) || ( FizzleEffect.bWasCompleted ) ) Destroy();

    }

	Super.Tick( FrameDelta );

    SetRotation( Rotation + Rotator( Normal( ProjectileVelocity * Vect( 1.0f, -1.0f, 0.0f ) ) ) );

}

Event PostBeginPlay()
{

    If( FireSound != None ) PlaySound( FireSound );

    SetTimer( FizzleTime, False, 'FizzleTimer' );

    Super.PostBeginPlay();

}

Event Touch( Actor Other, PrimitiveComponent OtherComponent, Vector HitLocation, Vector HitNormal )
{

    Local SPK_Ship Ship;

    If( HasTicked )
    {

        If( !Detonated )
        {

            If( Other != Owner )
            {

                If( SPK_PathProjectile( Other ) != None ) Return;

                Ship = SPK_Ship( Other );

                If( Ship != None )
                {

                    If( IgnoresInvincibility ) Ship.CauseDamageIgnoreInvincibility( Yield, Location ); Else Ship.CauseDamage( Yield, Location );

                }

                If( ( Ship != None ) || ( SPK_PathObject( Other ) != None ) || ( SPK_EnvironmentJunk( Other ) != None ) )
                {

                    RemoveVisibleComponents();
                    ClearTimer( 'FizzleTimer' );
                    Detonated = True;
                    If( HitSound != None ) PlaySound( HitSound );
                    If( HitEffect.Template != None ) HitEffect.ActivateSystem(); Else Destroy();

                }

            }

        }

    }

}

Function FizzleTimer()
{

        RemoveVisibleComponents();
        Detonated = True;
        If( FizzleSound != None ) PlaySound( FizzleSound );
        If( FizzleEffect.Template != None ) FizzleEffect.ActivateSystem(); Else Destroy();

}

function SetVelocity( vector Direction )
{

	ProjectileVelocity = Normal( Direction ) * InitialThrust;

}

Function RemoveVisibleComponents()
{

}

defaultproperties
{

	ProjectileVelocity = ( X = 0.0f, Y = 0.0f, Z = 0.0f );
	Thrust = 512.0f;
	InitialThrust = 1280.0f;
	Damping = 0.1f;

    FizzleTime = 3.0f;
    Detonated = False;

    FireSound = None;
    HitSound = None;
    FizzleSound = None;

    Yield = 5;

    IgnoresInvincibility = True;

    Begin Object Class=ParticleSystemComponent Name=Hit

        bAutoActivate = False;

    End Object

    Components.Add( Hit );

    HitEffect = Hit;

    Begin Object Class=ParticleSystemComponent Name=Fizzle

        bAutoActivate = False;

    End Object

    Components.Add( Fizzle );

    FizzleEffect = Fizzle;

    Begin Object Class=CylinderComponent Name=Cylinder

        CollideActors = True;

    End Object

    Components.Add( Cylinder );

    CollisionComponent = Cylinder;

    bCollideActors = True;

}
