class SPK_Ship extends SPK_PathObject;

var array<SPK_Hardpoint> Hardpoints;
var array<SPK_Hardpoint> HardpointsTick;
var array<SPK_Hardpoint> HardpointsFirePrimary;
var array<SPK_Hardpoint> HardpointsFireSecondary;
var array<SPK_Hardpoint> HardpointsGoLeft;
var array<SPK_Hardpoint> HardpointsGoRight;
var array<SPK_Hardpoint> HardpointsGoForwards;
var array<SPK_Hardpoint> HardpointsGoBackwards;
var array<SPK_Hardpoint> HardpointsStopLeft;
var array<SPK_Hardpoint> HardpointsStopRight;
var array<SPK_Hardpoint> HardpointsStopForwards;
var array<SPK_Hardpoint> HardpointsStopBackwards;
var array<SPK_Hardpoint> HardpointsDestroyed;
var array<SPK_Hardpoint> HardpointsDamaged;
var array<SPK_Hardpoint> HardpointsSpawned;

Var Int Health, HealthMax;
Var Float Shield, ShieldFade, ShieldTime;

Var Bool Left, Right, Forwards, Backwards;

Var Float Speed, SpeedMax, SpeedMin, SpeedNormal, SpeedDamping;
Var Float StrafeSpeed, StrafeDamping, StrafeAcceleration, StrafeRebound;

Var Bool Ambush;

/* Blaster selection variables. */
/* Limits the rate of fire and makes them alternate rather than firing at once. */
Var Bool PrimaryReady, Primary;
Var Float PrimaryInterval;
Var Int PrimarySelect;

Var SoundCue DeathSound;
Var ParticleSystemComponent DeathEffect;

/* Mass used for collision damage, relative to CruiserShip 1.0f. */
Var Float CollisionMass;

/* Post-damage invincibility. */
Var Float InvincibilityPeriod;

/* For the crumble effect on death. */
Var MaterialInstanceConstant Crumble;
Var Float CrumbleProgress;

Var Int KillScore;

Event Touch( Actor Other, PrimitiveComponent OtherComponent, Vector HitLocation, Vector HitNormal )
{

    Local SPK_Ship Ship;
    Local SPK_Game Game;

    If( HasTicked )
    {

        If( Health > 0 )
        {

            Ship = SPK_Ship( Other );

            If( Ship != None )
            {

                Game = SPK_Game( WorldInfo.Game );

                If( Game != None ) CauseDamage( Game.CollisionDamage * ( Ship.CollisionMass / ( CollisionMass + Ship.CollisionMass ) ), VLerp( Location, Other.Location, FRand() ) );

            }

        }

    }

}

Function SPK_Hardpoint AddHardpointAngle( Class<SPK_Hardpoint> Type, Vector Offset, Rotator Angle )
{

	Local SPK_Hardpoint Hardpoint;
    Local Vector XAxis, YAxis, ZAxis;

	Hardpoint = Spawn( Type, Self );

	if( Hardpoint != None )
	{

        GetAxes( Rotation, XAxis, YAxis, ZAxis );

        Hardpoint.SetRotation( Rotation + Angle );
		Hardpoint.SetLocation( Self.Location + ( XAxis * Offset.X ) + ( YAxis * Offset.Y ) + ( ZAxis * Offset.Z ) );
		Hardpoint.SetBase( Self );

        Hardpoint.Offset = Offset;

		Hardpoints[ Hardpoints.Length ] = Hardpoint;

	}

	return Hardpoint;

}

Function SPK_Hardpoint AddHardpoint( Class<SPK_Hardpoint> Type, Vector Offset )
{

	return AddHardpointAngle( Type, Offset, Rotator( Vect( 0.0f, 1.0f, 0.0f ) ) );

}

event Tick( float FrameDelta )
{

	local Rotator Test;

	local SPK_Hardpoint Hardpoint;

    if( Left ) StrafeAcceleration = Lerp( StrafeSpeed, StrafeAcceleration, StrafeDamping ** FrameDelta );
    if( Right ) StrafeAcceleration = Lerp( -StrafeSpeed, StrafeAcceleration, StrafeDamping ** FrameDelta );

    if( !( Left || Right ) ) StrafeAcceleration *= StrafeDamping ** FrameDelta;

    Strafe += StrafeAcceleration * FrameDelta;

    if( Strafe > 96.0f )
    {

        Strafe = 96.0f;
        StrafeAcceleration = Abs( StrafeAcceleration ) * -StrafeRebound;

    }

    if( Strafe < -96.0f )
    {

        Strafe = -96.0f;
        StrafeAcceleration = Abs( StrafeAcceleration ) * StrafeRebound;

    }

    Foreach HardpointsTick( Hardpoint )
    {

        Hardpoint.Trigger( SPK_HARDPOINTEVENT_TICK );

    }

    Test = Self.Rotation;
    Test.Yaw += FrameDelta * 32768;

    SetRotation( Test );

    if( Forwards ) Speed = Lerp( SpeedMax, Speed, SpeedDamping ** FrameDelta );
    if( Backwards ) Speed = Lerp( SpeedMin, Speed, SpeedDamping ** FrameDelta );

    if( !( Forwards || Backwards ) ) Speed = Lerp( SpeedNormal, Speed, SpeedDamping ** FrameDelta );

    If( Ambush ) Progress = AttachedTo.Progress( Progress, FrameDelta * Speed ); Else Progress = AttachedTo.Progress( Progress, FrameDelta * -Speed );

    If( Health <= 0 )
    {

        CrumbleProgress += FrameDelta;

        If( CrumbleProgress > 1.0f )
        {

            If( Crumble != None ) Crumble.SetScalarParameterValue( 'Crumble', 1.0f );

            If( DeathEffect.Template == None ) Destroy(); Else If( DeathEffect.bWasCompleted ) Destroy();

        }
        Else If( Crumble != None ) Crumble.SetScalarParameterValue( 'Crumble', CrumbleProgress );

    }

	Super.Tick( FrameDelta );

    If( !Ambush ) SetRotation( Rotator( Vect( -1.0f, 0.0f, 0.0f ) ) + Rotation );

}

Function PrimaryReloaded()
{

	if( Primary )
	{

		PrimaryFired();

	} else PrimaryReady = True;

}

Function FirePrimary()
{

	if( PrimaryReady )
	{

		Primary = True;
		PrimaryFired();

	}

}

Function PrimaryFired()
{

	if( HardpointsFirePrimary.Length > 0 )
	{

		PrimarySelect++;
		PrimarySelect = PrimarySelect % HardpointsFirePrimary.Length;

		HardpointsFirePrimary[ PrimarySelect ].Trigger( SPK_HARDPOINTEVENT_FIREPRIMARY );

	}

	PrimaryReady = False;

	SetTimer( PrimaryInterval, False, 'PrimaryReloaded' );

}

Function CeasePrimary()
{

	Primary = False;

}

Function FireSecondary()
{

	Local SPK_Hardpoint Hardpoint;

	ForEach HardpointsFireSecondary( Hardpoint ) Hardpoint.Trigger( SPK_HARDPOINTEVENT_FIRESECONDARY );

}

Event Destroyed()
{

    Local SPK_Hardpoint Cleanup;

    ForEach Hardpoints( Cleanup ) Cleanup.Destroy();
    Hardpoints.Remove( 0, Hardpoints.Length );

    Super.Destroyed();

}

Function Die()
{

    Local SPK_Game GameType;
    Local SPK_PlayerController Game;
    Local SPK_Hardpoint Cleanup;

    GameType = SPK_Game( WorldInfo.Game );
    Game = SPK_PlayerController( Owner );

    Health = 0;

    ForEach Hardpoints( Cleanup ) Cleanup.Destroy();
    Hardpoints.Remove( 0, Hardpoints.Length );

    Crumble = New( None ) Class'MaterialInstanceConstant';

    If( Crumble != None )
    {

        Crumble.SetParent( StaticMeshComponent( CollisionComponent ).GetMaterial( 0 ) );

        StaticMeshComponent( CollisionComponent ).SetMaterial( 0, Crumble );

    }

    If( Game != None ) If( Self == Game.Control ) Game.PlayerDied( Progress );

    If( GameType != None ) GameType.Score += KillScore;

    If( DeathSound != None ) PlaySound( DeathSound );
    If( DeathEffect.Template == None ) Destroy(); Else DeathEffect.ActivateSystem();

}

Function CauseDamageIgnoreInvincibility( Int Amount, Vector Where )
{

    Local SPK_Hardpoint Cleanup;

    ForEach HardpointsDamaged( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_DAMAGED );

    If( Health > 0 )
    {

        Health -= Amount;

        If( Health <= 0 )
        {

            Die();

        }

    }

}

Function CauseDamage( Int Amount, Vector Where )
{

    Local SPK_PlayerController Game;
    Local SPK_Hardpoint Cleanup;

    If( !IsTimerActive( 'InvincibilityTimer' ) )
    {

        ForEach HardpointsDamaged( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_DAMAGED );

        If( Health > 0 )
        {

            Game = SPK_PlayerController( Owner );

            Health -= Amount;

            If( Health <= 0 )
            {

                Die();

            }
            Else If( Game != None ) If( Self == Game.Control ) MakeInvincible();

        }

    }

}

Function MakeInvincible()
{

    If( !IsTimerActive( 'InvincibilityTimer' ) )
    {

        If( InvincibilityPeriod > 0.0f )
        {

            InvincibilityFlickerOff();
            SetTimer( InvincibilityPeriod, False, 'InvincibilityTimer' );

        }

    }

}

Function InvincibilityTimer()
{

    ClearTimer( 'InvincibilityFlickerOn' );
    ClearTimer( 'InvincibilityFlickerOff' );
    SetHidden( False );

}

Function InvincibilityFlickerOff()
{

    SetHidden( True );
    SetTimer( 0.03f, False, 'InvincibilityFlickerOn' );

}

Function InvincibilityFlickerOn()
{

    SetHidden( False );
    SetTimer( 0.03f, False, 'InvincibilityFlickerOff' );

}

Function GoLeft()
{

    Local SPK_Hardpoint Cleanup;

    If( Right ) ForEach HardpointsStopRight( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_STOPRIGHT );

    ForEach HardpointsGoLeft( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_GOLEFT );

	Right = False;
	Left = True;

}

Function GoRight()
{

    Local SPK_Hardpoint Cleanup;

    If( Left ) ForEach HardpointsStopLeft( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_STOPLEFT );

    ForEach HardpointsGoLeft( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_GORIGHT );

	Left = False;
	Right = True;

}

Function GoForwards()
{

    Local SPK_Hardpoint Cleanup;

    If( Backwards ) ForEach HardpointsStopForwards( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_STOPBACKWARDS );

    ForEach HardpointsGoForwards( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_GOFORWARDS );

	Backwards = False;
	Forwards = True;

}

Function GoBackwards()
{

    Local SPK_Hardpoint Cleanup;

    If( Forwards ) ForEach HardpointsStopForwards( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_STOPFORWARDS );

    ForEach HardpointsGoForwards( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_GOBACKWARDS );

	Forwards = False;
	Backwards = True;

}

Function StopLeft()
{

    Local SPK_Hardpoint Cleanup;

    If( Left ) ForEach HardpointsStopLeft( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_STOPLEFT );
    If( Right ) ForEach HardpointsStopRight( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_STOPRIGHT );

	Left = False;
	Right = False;

}

Function StopRight()
{

    Local SPK_Hardpoint Cleanup;

    If( Left ) ForEach HardpointsStopLeft( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_STOPLEFT );
    If( Right ) ForEach HardpointsStopRight( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_STOPRIGHT );

	Left = False;
	Right = False;

}

Function StopForwards()
{

    Local SPK_Hardpoint Cleanup;

    If( Backwards ) ForEach HardpointsStopForwards( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_STOPBACKWARDS );
    If( Forwards ) ForEach HardpointsStopForwards( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_STOPFORWARDS );

	Forwards = False;
	Backwards = False;

}

Function StopBackwards()
{

    Local SPK_Hardpoint Cleanup;

    If( Backwards ) ForEach HardpointsStopForwards( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_STOPBACKWARDS );
    If( Forwards ) ForEach HardpointsStopForwards( Cleanup ) Cleanup.Trigger( SPK_HARDPOINTEVENT_STOPFORWARDS );

	Forwards = False;
	Backwards = False;

}

defaultproperties
{

	Health = 100;
	HealthMax = 100;

	Shield = -1.0f;
	ShieldFade = -1.0f;
	ShieldTime = 2.4f;

	Left = False;
	Right = False;
	Forwards = False;
	Backwards = False;

	SpeedMax = 320.0f;
	SpeedMin = 96.0f;
	SpeedDamping = 0.2f;
	SpeedNormal = 192.0f;
	Speed = 0.0f;

	StrafeSpeed = 512.0f;
	StrafeDamping = 0.2f;
	StrafeAcceleration = 0.0f;
	StrafeRebound = 0.1f;

	Primary = False;
	PrimaryReady = True;
	PrimaryInterval = 0.35f;
	PrimarySelect = 0;

    CollisionMass = 1.0f;

    InvincibilityPeriod = -1.0f;

    Crumble = None;
    CrumbleProgress = 0.0f;

    KillScore = 0;

    DeathSound = SoundCue'SPK_Hardpoints.Explosion_Big';

    Begin Object Class=ParticleSystemComponent Name=Death

        bAutoActivate = False;

        Template = ParticleSystem'SPK_Hardpoints.Explosion_Huge';

    End Object

    Components.Add( Death );

    DeathEffect = Death;

	Begin Object Class=DynamicLightEnvironmentComponent Name=DynamicLightEnvironmentComponent0

		bEnabled = True;

	End Object

	Components.Add( DynamicLightEnvironmentComponent0 );

    Begin Object Class=StaticMeshComponent Name=StaticMeshComponent0

		LightEnvironment = DynamicLightEnvironmentComponent0;
        CollideActors = True;

	End Object

	Components.Add( StaticMeshComponent0 );

    CollisionComponent = StaticMeshComponent0;

    bCollideActors = True;

}
